{******************************************************************************}
{******************************************************************************}
{*** Unit:   PopButton                                                      ***}
{*** Author: Bas Weerman. CentERdata, Tilburg University                    ***}
{*** Date:   23/12/1997                                                     ***}
{*** Uses:   Messages, Classes, Graphics, Controls en ExtCtrls              ***}
{***                                                                        ***}
{*** Description:                                                           ***}
{*** This unit simulates the 'new' popping buttons, used by (e.g.)microsoft ***}
{*** office 97 and the netscape navigator. It is also possible to put       ***}
{*** images on the popbutton.                                               ***}
{***                                                                        ***}
{*** Features:
{*** property FocusColor    : Sets the Color of the Text while the button   ***}
{***                          has the focus.                                ***}
{*** property PressedColor  : Sets the Color of the Text while the button   ***}
{***                          is pressed.                                   ***}
{*** property OnGotFocus    : Calls the GotFocus Event                      ***}
{*** property OnLostFocus   : Calls the LostFocus Event                     ***}
{***                                                                        ***}
{*** It is possible to drop TImage objects on the PopButton. They will then ***}
{*** also raise the OnClick event of the PopButton.                         ***}
{******************************************************************************}
{******************************************************************************}
unit PopButton;

interface

uses
  Messages, Classes, Graphics, Controls, ExtCtrls;

type
  {*** Two new focusevents ***}
  TGotFocusEvent  = procedure(Sender: TObject) of object;
  TLostFocusEvent = procedure(Sender: TObject) of object;

  TPopButton = class(TPanel)
   private
    FInit          : Boolean;
    SavedFontColor : TColor;
    FPressedColor  : TColor;
    FFocusColor    : TColor;
    FOnGotFocus    : TGotFocusEvent;
    FOnLostFocus   : TLostFocusEvent;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure Init;
  protected
    procedure GotFocus; dynamic;
    procedure LostFocus; dynamic;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure ControlMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ControlMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ControlClick (Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FocusColor   : TColor          read FFocusColor
                                            write FFocusColor;
    property PressedColor : TColor          read FPressedColor
                                            write FPressedColor;
    property OnGotFocus   : TGotFocusEvent  read FOnGotFocus
                                            write FOnGotFocus;
    property OnLostFocus  : TLostFocusEvent read FOnLostFocus
                                            write FOnLostFocus;
  end;

procedure Register;

implementation

procedure Register;
begin
  {*** Register this component. Add it to Samples ***}
  RegisterComponents('Samples', [TPopButton]);
end;

procedure TPopButton.GotFocus;
begin
  Init; {*** Init the PopButton during the first focus ***}
  Font.Color := FocusColor;
  BevelInner := bvRaised;
  BevelOuter := bvRaised;
  if Assigned(FOnGotFocus) then FOnGotFocus(Self);
end;

procedure TPopButton.LostFocus;
begin
  Font.Color := SavedFontColor;
  BevelInner := bvNone;
  BevelOuter := bvNone;
  if Assigned(FOnLostFocus) then FOnLostFocus(Self);
end;

procedure TPopButton.CMMouseEnter(var Message: TMessage);
begin
  with Message do GotFocus;
end;

procedure TPopButton.CMMouseLeave(var Message: TMessage);
begin
  with Message do LostFocus;
end;

procedure TPopButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
begin
 inherited MouseUp (Button, Shift, X, Y);
 {* If the focus is lost, then the bevels should not be raised! ***}
 if not (BevelOuter = bvNone) then begin
     Font.Color := FocusColor;
     BevelInner := bvRaised;
     BevelOuter := bvRaised;
    end;
end;

procedure TPopButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
begin
 inherited MouseDown (Button, Shift, X, Y);
 Font.Color := PressedColor;
 BevelInner := bvLowered;
 BevelOuter := bvLowered;
end;

constructor TPopButton.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 {*** Save the FontColor, so it can be changed to its original color after
      a lostfocus event ***}
 SavedFontColor := Font.Color;
 BevelInner := bvNone;
 BevelOuter := bvNone;
 FInit      := true;
end;

procedure TPopButton.Init;
var I: Integer;
begin
 {*** If there are TImage objects on the PopButton, the down , up and click
 {*** events should be redirected ***}
 if FInit then {*** Only if it is the first time ***}
 for I:= 0 to ControlCount -1 do
     if Controls[I] is TImage then begin
        TImage(Controls[I]).OnMouseDown := ControlMouseDown;
        TImage(Controls[I]).OnMouseup   := ControlMouseUp;
        TImage(Controls[I]).OnClick     := ControlClick;
     end;
  FInit := false; {*** Go through Init just one time ***}
end;

destructor TPopButton.Destroy;
begin
 {*** Anything to throw out?? ***}
 inherited Destroy;
end;

procedure TPopButton.ControlMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
begin
 {*** Redirect the MouseUp event ***}
 MouseUp(Button, Shift, X, Y);
end;

procedure TPopButton.ControlMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
begin
 {*** Redirect the MouseDown event ***}
 MouseDown(Button, Shift, X, Y);
end;

procedure TPopButton.ControlClick (Sender: TObject);
begin
 {*** Redirect the MouseDown event ***}
 Click;
end;

end.
